// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * At most one train assigned per departure.
 *
 * For any departure, there must not be two different trains with a departure
 * event associated.
 *
 * @author YDS
 *
 */
public class AtMostOneTrainPerDepartureConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public AtMostOneTrainPerDepartureConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        HashMap<Departure, Train> assigned =
                new HashMap<Departure, Train>();
        TrainDAO dao = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.Departure)) {
                    Departure dep = departureDAO.loadByName(
                            fill.getComplement());
                    if (dep != null) {
                        Train another = assigned.put(dep, train);
                        if (another != null) {
                            logger.log(Level.INFO, "At most one train "
                                    + "assigned per departure error : "
                                    + train.getName() + " is assigned with "
                                    + dep.getName() + " already covered by "
                                    + another.getName() + ".");
                            valid = false;
                        }
                    }
                }
            }
        }

        return valid;
    }

}
