package com.sncf.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Resource;
import com.sncf.checker.bo.TrackGroup;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.ResourceDAO;

public class ExitFromResourceAfterDisjunctionConstraint implements Constraint {

    /** Interface to Data layer. */
    private final DAOFactory factory;

    public ExitFromResourceAfterDisjunctionConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        boolean valid = true;
        ResourceDAO resourceDAO = factory.getResourceDAO();
        Logger logger = Logger.getLogger("logger");

        for (Resource resource : resourceDAO.loadAll()) {
            if (resource.getClass() != TrackGroup.class) {
                List<Fill> fillList = resource.getFill();
                Collections.sort(fillList, new SortFillByDate());

                for (int i = 0; i < fillList.size(); i++) {
                    Fill disjunction = fillList.get(i);
                    if (disjunction.getEvent().equals(Event.BegDisjunction)) {
                        ArrayList<Fill> exitTimes = new ArrayList<Fill>();
                        ArrayList<String> assembledTrains = new ArrayList<String>(
                                Arrays.asList(disjunction.getComplement().split("\\+")));
                        for (String train : assembledTrains) {
                            for (int j = i + 1; j < fillList.size(); j++) {
                                Fill fill = fillList.get(j);
                                if (fill.getTrain().getName().equals(train)) {
                                    if (fill.getEvent().equals(Event.ExitResource)) {
                                        exitTimes.add(fill);
                                        break;
                                    }
                                }
                            }
                        }
                        for (int j = 0; j < exitTimes.size() - 1; j++) {
                            for (int k = j + 1; k < exitTimes.size(); k++) {
                                Fill f1 = exitTimes.get(j);
                                Fill f2 = exitTimes.get(k);
                                boolean correct = true;
                                if (f1.getGate().length() >= 1
                                        && f2.getGate().length() >= 1) {

                                    if (f1.getGate().substring(0, 1).equals("B")
                                            && f2.getGate().substring(0, 1).equals("A")) {
                                        correct = false;
                                    } else if (f1.getGate().substring(0, 1).equals("A")
                                            && f2.getGate().substring(0, 1).equals("A")
                                            && f1.getDate().getTime() > f2.getDate().getTime()) {
                                        correct = false;
                                    } else if (f1.getGate().substring(0, 1).equals("B")
                                            && f2.getGate().substring(0, 1).equals("B")
                                            && f1.getDate().getTime() < f2.getDate().getTime()) {
                                        correct = false;
                                    }
                                } else if (f1.getGate().length() == 0
                                        && f2.getGate().length() >= 1) {
                                    if (f2.getGate().substring(0, 1).equals("A")) {
                                        correct = false;
                                    }
                                } else if (f1.getGate().length() >= 1
                                        && f2.getGate().length() == 0) {
                                    if (f1.getGate().substring(0, 1).equals("B")) {
                                        correct = false;
                                    }
                                }
                                if (!correct) {
                                    logger.log(Level.INFO, "Exit from Resource after disjunction error : "
                                            + f1.getTrain().getName() + " (" + f1.getDate().toString()
                                            + " " + f1.getGate()
                                            + ") is in conflict with "
                                            + f2.getTrain().getName() + " (" + f2.getDate().toString()
                                            + " " + f2.getGate() + ") at " + resource.getName() + ".");
                                    valid = false;
                                }
                            }
                        }
                    }

                }
            }
        }
        return valid;
    }

}
