// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Facility;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Parameters;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.MaintenanceFacilityDAO;

/**
 * Capacity of maintenance resources.
 *
 * @author YDS
 *
 */
public class MaintenanceResourcesCapacityConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public MaintenanceResourcesCapacityConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public boolean validate() throws ROADEFException {
        MaintenanceFacilityDAO dao = factory.getMaintenanceFacilityDAO();
        Parameters parameters = factory.getParametersDAO().getParameters();
        int maxMaint = parameters.getMaxMaint();
        boolean valid = true;
        int[] nbMaintenance = new int[parameters.getNbDays()];
        Logger logger = Logger.getLogger("logger");

        for (Facility facility : dao.loadAll()) {
            for (Fill fill : facility.getFill()) {
                if (fill.getEvent().equals(Event.BegMaintenance)) {
                    nbMaintenance[fill.getDate().getDay() - 1]++;
                }
            }
        }

        for (int i = 1; i <= nbMaintenance.length; i++) {
            if (nbMaintenance[i - 1] > maxMaint) {
                logger.log(Level.INFO, "Capacity of maintenance resources : "
                        + "at most " + maxMaint + " maintenance operations "
                        + "may start on day " + i + " (" + nbMaintenance[i - 1]
                        + " found).");
                valid = false;
            }
        }

        return valid;
    }

}
