// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Parameters;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.ArrivalDAO;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.FillDAO;

public class PreferredPlatformAssignementCost implements Cost {

    private final DAOFactory factory;

    public PreferredPlatformAssignementCost(DAOFactory factory) {
        this.factory = factory;
    }

    public float calculate() throws ROADEFException {
        FillDAO fillDAO = factory.getFillDAO();
        ArrivalDAO arrivalDAO = factory.getArrivalDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        Parameters parameters = factory.getParametersDAO().getParameters();
        float score = 0;
        Logger logger = Logger.getLogger("logger");

        for (Fill fill : fillDAO.loadAll()) {
            if (fill.getEvent().equals(Event.Arrival)) {
                Arrival arr = arrivalDAO.loadByName(fill.getComplement());
                if (arr != null) {
                    if (!arr.getPreferedPlatforms().contains(fill.getResource())) {
                        logger.log(Level.INFO, "Preferred platform assignement cost : "
                                + fill.getTrain().getName()
                                + " does not satify preferred platform ("
                                + fill.getResource().getName() + ") assignment for "
                                + arr.getName() + ".  Score += "
                                + parameters.getPlatAsgCost());
                        score += parameters.getPlatAsgCost();
                    }
                }
            } else if (fill.getEvent().equals(Event.Departure)) {
                Departure dep = departureDAO.loadByName(fill.getComplement());
                if (dep != null) {
                    if (!dep.getPreferedPlatforms().contains(fill.getResource())) {
                        logger.log(Level.INFO, "Preferred platform assignement cost : "
                                + fill.getTrain().getName()
                                + " does not satify preferred platform ("
                                + fill.getResource().getName() + ") assignment for "
                                + dep.getName() + ". Score += "
                                + parameters.getPlatAsgCost());
                        score += parameters.getPlatAsgCost();
                    }
                }
            }
        }
        return score;
    }

}
