// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Resource;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.ResourceDAO;

public class ResourceTrainCategoryCompatibilityConstraint implements Constraint {

    private DAOFactory factory;

    public ResourceTrainCategoryCompatibilityConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        ResourceDAO dao = factory.getResourceDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Resource> resourceList = dao.loadAll();
        for (Resource resource : resourceList) {
            for (Fill fill : resource.getFill()) {
                Train train = fill.getTrain();
                if (!resource.getCompCatRes().contains(train.getCategory())) {
                    logger.log(Level.INFO, "Resource - train category compatibility error : "
                            + train.getName() + " (" + train.getCategory().getName()
                            + ") is not compatible with " + resource.getName() + " at "
                            + fill.getDate().toString() + ".");
                    valid = false;
                }
            }
        }
        return valid;
    }

}
