// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Parameters;
import com.sncf.checker.bo.Reuse;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.ReuseDAO;

public class TrainReuseCost implements Cost {

    private final DAOFactory factory;

    public TrainReuseCost(DAOFactory factory) {
        this.factory = factory;
    }

    public float calculate() throws ROADEFException {
        ReuseDAO reuseDAO = factory.getReuseDAO();
        Parameters parameters = factory.getParametersDAO().getParameters();
        float score = 0;
        Logger logger = Logger.getLogger("logger");

        for (Reuse reuse : reuseDAO.loadAll()) {
            boolean satisfied = false;
            for (Fill fill : reuse.getArrival().getTrain().getFill()) {
                if (fill.getEvent().equals(Event.Departure)) {
                    if (fill.getComplement().equals(reuse.getDeparture().getName())) {
                        satisfied = true;
                    }
                }
            }
            if (!satisfied) {
                logger.log(Level.INFO, "Train reuse cost : "
                        + reuse.getArrival().getName() + " - "
                        + reuse.getDeparture().getName()
                        + " reuse is not satified.  Score += "
                        + parameters.getReuseCost());
                score += parameters.getReuseCost();
            }
        }
        return score;
    }
}
