// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Gate;
import com.sncf.checker.bo.TrackGroup;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

public class TransitionConstraint implements Constraint {

    private final DAOFactory factory;

    public TransitionConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            Fill previousExit = null;
            for (Fill fill : train.getFill()) {
                if (previousExit == null) {
                    if (fill.getEvent().equals(Event.ExitResource)) {
                        previousExit = fill;
                    }
                } else {
                    if (fill.getEvent().equals(Event.EnterResource)) {
                        boolean isTransitionCorrect = false;
                        for (Gate gate : previousExit.getResource().getGates()) {
                            if (previousExit.getGate().equals(gate.getSide() + gate.getIndex())
                                && fill.getGate().equals(gate.getNeighborSide() + gate.getNeighborIndex())
                                && fill.getResource() == gate.getNeighborResource()) {
                                isTransitionCorrect = true;
                                break;
                            }
                        }
                        if (!isTransitionCorrect) {
                            logger.log(Level.INFO, "Transition error : "
                                + train.getName() + " has an incorrect "
                                + "transition from "
                                + previousExit.getResource().getName()
                                + " " + previousExit.getGate() + " to "
                                + fill.getResource().getName() + " "
                                + fill.getGate() + " at "
                                + fill.getDate().toString() + ".");
                            valid = false;
                        }
                    } else if (fill.getEvent().equals(Event.ExitSystem)) {
                        boolean isTransitionCorrect = false;
                        if (previousExit != null
                                && previousExit.getResource().getClass()
                                == TrackGroup.class) {
                            Gate exitGate = null;
                            for (Gate gate : previousExit.getResource().getGates()) {
                                if (previousExit.getGate().equals(gate.getSide() + gate.getIndex())) {
                                    exitGate = gate;
                                    isTransitionCorrect = true;
                                    break;
                                }
                            }
                            if (!isTransitionCorrect) {
                                logger.log(Level.INFO, "Transition error : "
                                    + train.getName() + " has an incorrect "
                                    + "gate at"
                                    + previousExit.getResource().getName()
                                    + " (" + previousExit.getGate() + " does "
                                    + "not exist).");
                                valid = false;
                            }
                        }
                    } else {
                        logger.log(Level.INFO, "Transition error : "
                            + train.getName() + " EnterResource event expected"
                            + " after an ExitResource event at "
                            + fill.getDate().toString() + " ("
                            + fill.getEvent().toString() + " found).");
                        valid = false;
                    }
                    previousExit = null;
                }
            }
        }
        return valid;
    }

}
