// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.bo;

import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.FillDAO;

/**
 * physical trains (rolling stock).
 *
 * @author YDS
 *
 */
public class Train {

    /** dao for fill list lazy loading. */
    private FillDAO fillDAO;

    /**
     *  constructor.
     */
    public Train() {
        fillList = new ArrayList<Fill>();
    }

    /**
     * lazy-loading version constructor.
     */
    public Train(final FillDAO fillDAO) {
        fillList = null;
        this.fillDAO = fillDAO;
    }

    /** name. */
    private String name;

    /**
     * Get the name.
     *
     * @return name
     */
    public final String getName() {
        return name;
    }

    /**
     * Set the name.
     *
     * @param name String to set as name
     */
    public final void setName(final String name) {
        this.name = name;
    }

    /** initial potential in km. */
    private int remDBM;

    /**
     * Get the the initial potential in km.
     *
     * @return initial potential in km
     */
    public final int getRemDBM() {
        return remDBM;
    }

    /**
     * Set the initial potential in km.
     *
     * @param initialPotKm
     */
    public final void setRemDBM(final int remDBM) {
        this.remDBM = remDBM;
    }

    /** initial potential in time. */
    private Duration remTBM;

    /**
     * Get the initial potential in time.
     *
     * @return initial potential in time
     */
    public final Duration getRemTBM() {
        return remTBM;
    }

    /**
     * Set the initial potential in time.
     *
     * @param duration
     */
    public final void setRemTBM(final Duration duration) {
        this.remTBM = duration;
    }

    /** fill list. */
    private ArrayList<Fill> fillList;

    /**
     * Get the occupation list.
     *
     * @return occupation list
     * @throws ROADEFException
     */
    public final List<Fill> getFill() throws ROADEFException {
        if (fillList == null) {
            fillList = new ArrayList<Fill>(fillDAO.loadByTrain(name));
        }
        return fillList;
    }

    /**
     * Add an occupation at given time.
     *
     * @param fill
     * @throws ROADEFException
     */
    public final void addFill(final Fill fill) throws ROADEFException {
        if (fillList == null) {
            fillList = new ArrayList<Fill>(fillDAO.loadByTrain(name));
        }
        fillList.add(fill);
    }

    /** category. */
    private TrainCategory category;

    /**
     * Get the category.
     *
     * @return TrainCategory
     */
    public final TrainCategory getCategory() {
        Arrival arr = getArrival();
        if (arr != null) {
            Departure dep = arr.getLinkedDep();
            if (dep != null) {
                Train prev = dep.getTrain();
                if (prev != null) {
                    return prev.getCategory();
                }
            }
        }
        return category;
    }

    /**
     * Set the category.
     *
     * @param TrainCategory
     */
    public final void setCategory(final TrainCategory category) {
        this.category = category;
    }

    /** arrival. */
    private Arrival arrival;

    /**
     * Get the arrival.
     *
     * @return Arrival
     */
    public final Arrival getArrival() {
        return arrival;
    }

    /**
     * Set the arrival.
     *
     * @param arrival Arrival to set
     */
    public final void setArrival(final Arrival arrival) {
        this.arrival = arrival;
        if (arrival != null) {
            if (arrival.getTrain() != this) {
                arrival.setTrain(this);
            }
        }
    }

    /** departure. */
    private Departure departure;

    /**
     * Get the departure.
     *
     * @return Departure
     */
    public final Departure getDeparture() {
        return departure;
    }

    /**
     * Set the departure.
     *
     * @param departure Departure to set
     */
    public final void setDeparture(final Departure departure) {
        this.departure = departure;
        if (departure != null) {
            if (departure.getTrain() != this) {
                departure.setTrain(this);
            }
        }
    }

    /** related occupation. */
    private Resource resource;

    /**
     * Get the related occupation.
     *
     * @return occupation
     */
    public final Resource getResource() {
        return resource;
    }

    /**
     * Set the related occupation.
     *
     * @param occupation
     */
    public final void setResource(Resource resource) {
        this.resource = resource;
    }
}
