// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.bo.Resource;

/**
 * Resource objects do not exist in the scenario files. Loaders to corresponding classes are called.
 *
 * @author YDS
 *
 */
public class ResourceFlatFileDAO implements ResourceDAO {

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public ResourceFlatFileDAO(final String scenario, final String solution) {
    }

    /** Yard DAO. */
    private YardDAO yardDAO;

    /**
     * Yard DAO setter.
     *
     * @param dao Yard DAO
     */
    public void setYardSequenceDAO(final YardDAO dao) {
        this.yardDAO = dao;
    }

    /** TrackGroup DAO. */
    private TrackGroupDAO trackGroupDAO;

    /**
     * TrackGroup DAO setter.
     *
     * @param dao TrackGroup DAO
     */
    public void setTrackGroupDAO(final TrackGroupDAO dao) {
        this.trackGroupDAO = dao;
    }

    /** IndividualTrack DAO. */
    private IndividualTrackDAO individualTrackDAO;

    /**
     * IndividualTrack DAO setter.
     *
     * @param dao IndividualTrack DAO
     */
    public void setIndividualTrackDAO(final IndividualTrackDAO dao) {
        this.individualTrackDAO = dao;
    }

    /**
     * Load a resource track by name.
     *
     * @param name the searched resource name
     * @return resource object, null if not found
     */
    public Resource loadByName(final String name) {
        Resource entity = null;

        entity = trackGroupDAO.loadByName(name);
        if (entity != null) {
            return entity;
        }
        entity = yardDAO.loadByName(name);
        if (entity != null) {
            return entity;
        }
        entity = individualTrackDAO.loadByName(name);
        if (entity != null) {
            return entity;
        }
        return entity;
    }

    /**
     * Load all resource.
     *
     * @return resource list
     */
    public List<Resource> loadAll() {
        ArrayList<Resource> list = new ArrayList<Resource>();

        list.addAll(trackGroupDAO.loadAll());
        list.addAll(yardDAO.loadAll());
        list.addAll(individualTrackDAO.loadAll());
        return list;
    }

}
