// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.Reuse;

public class ReuseFlatFileDAO extends FlatFileDAO implements ReuseDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name train map of the scenario. */
    private List<Reuse> cache = null;

    public ReuseFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** Arrival DAO. */
    private ArrivalDAO arrivalDAO;

    /**
     * Arrival DAO setter.
     *
     * @param dao Arrival DAO
     */
    public void setArrivalDAO(final ArrivalDAO dao) {
        this.arrivalDAO = dao;
    }

    /** Departure DAO. */
    private DepartureDAO departureDAO;

    /**
     * Departure DAO setter.
     *
     * @param dao Departure DAO
     */
    public void setDepartureDAO(final DepartureDAO dao) {
        this.departureDAO = dao;
    }

    public List<Reuse> loadAll() {
        initialize();
        ArrayList<Reuse> list = new ArrayList<Reuse>();
        list.addAll(cache);
        return list;
    }

    /**
     * Extract all trains from initialTrains.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new ArrayList<Reuse>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "reuses.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";", -1);
                    Reuse temp = new Reuse();

                    cache.add(temp);
                    temp.setArrival(arrivalDAO.loadByName(splits[0]));
                    temp.setDeparture(departureDAO.loadByName(splits[1]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}
